use bevy::prelude::*;
use bevy_eventwork::{tcp::TcpProvider, AppNetworkMessage, NetworkData};

use termion::{clear, cursor::Goto};

use crate::common::{self, server_msg::MapUpdates};

pub struct DisplayPlugin;

impl Plugin for DisplayPlugin {
	fn build(&self, app: &mut App) {
		app.listen_for_message::<MapUpdates, TcpProvider>()
			.add_systems(Update, handle_map_updates);
	}
}

struct Display;

impl Display {
	fn draw(&self, pos: IVec2, (l, r): (char, char)) {
		let pos = pos + IVec2::new(RENDER_DISTANCE, RENDER_DISTANCE);
		let x = (pos.x * 2) + 1;
		let y = ((2 * RENDER_DISTANCE) - pos.y) + 1;
		print!("{}{}{}", Goto(x as _, y as _), l, r);
	}
}

const RENDER_DISTANCE: i32 = common::RENDER_DISTANCE;
const RENDER_DISTANCE_SQ: i32 = RENDER_DISTANCE * RENDER_DISTANCE;

fn handle_map_updates(mut updates: EventReader<NetworkData<MapUpdates>>) {
	for update in updates.read() {
		print!("{}", clear::All);
		for x in -RENDER_DISTANCE..RENDER_DISTANCE {
			for y in -RENDER_DISTANCE..RENDER_DISTANCE {
				let pos = IVec2::new(x, y);
				if pos.distance_squared(IVec2::ZERO) <= RENDER_DISTANCE_SQ {
					Display.draw(pos, ('.', ' '));
				}
			}
		}

		for (pos, chars) in &update.0 {
			Display.draw(*pos, *chars);
		}
		println!();
	}
}
