use bevy::{
	prelude::*,
	tasks::{TaskPool, TaskPoolBuilder},
};
use bevy_eventwork::{
	tcp::{NetworkSettings, TcpProvider},
	ConnectionId, EventworkPlugin, EventworkRuntime, Network, NetworkEvent,
};
use termion::clear;

pub struct NetPlugin;

impl Plugin for NetPlugin {
	fn build(&self, app: &mut App) {
		app.add_plugins(EventworkPlugin::<TcpProvider, TaskPool>::default())
			.insert_resource(EventworkRuntime(TaskPoolBuilder::new().num_threads(2).build()))
			.insert_resource(NetworkSettings::default())
			.add_systems(Update, handle_net_event)
			.init_resource::<Server>()
			.add_systems(Startup, do_connect);
	}
}

#[derive(Debug, Resource, Default)]
pub struct Server(pub Option<ConnectionId>);

fn handle_net_event(mut events: EventReader<NetworkEvent>, mut server: ResMut<Server>) {
	for event in events.read() {
		match event {
			NetworkEvent::Connected(connection_id) => {
				println!("Connected '{connection_id:?}'.");
				server.0 = Some(*connection_id);
			}
			NetworkEvent::Disconnected(connection_id) => {
				print!("{}", clear::All);
				println!("Disconnected '{connection_id:?}'.");
				panic!();
			}
			NetworkEvent::Error(network_error) => println!("Error '{network_error}'."),
		}
	}
}

fn do_connect(
	net: Res<Network<TcpProvider>>,
	settings: Res<NetworkSettings>,
	task_pool: Res<EventworkRuntime<TaskPool>>,
) {
	net.connect(([0, 0, 0, 0], 9000).into(), &task_pool.0, &settings);
}
