use std::ops::Not;

use bevy::prelude::*;
use itermore::IterArrayChunks;

use super::{
	display::Display,
	physics::{Mass, Pos},
};

fn spawn_block(cmd: &mut Commands, pos: IVec2, display: (char, char), mass: usize) {
	cmd.spawn((Pos(pos.as_vec2()), Mass(mass), Display(display)));
}

fn str_to_struct(text: &str) -> Vec<(IVec2, (char, char), usize)> {
	text.lines()
		.enumerate()
		.flat_map(|(y, line)| {
			line.chars().array_chunks().enumerate().filter_map(move |(x, [l, r])| {
				(l == ' ' && r == ' ')
					.not()
					.then_some((IVec2::new(x as _, y as _), (l, r), mass_of((l, r))))
			})
		})
		.collect()
}

fn mass_of(tiles: (char, char)) -> usize {
	match tiles {
		('#', '#') => 100,
		('X', 'X') => 10,
		('[', ']') => 5,
		('(', ')') => 1,
		_ => 20,
	}
}

pub fn spawn_debug_map(mut cmd: Commands) {
	let raw = "
        ()            
                      
      ##########  ()  
    ()##      ##      
      ##  []          
      ##      ##      
      ##########()    
                      
                      
";

	for (pos, display, mass) in str_to_struct(raw) {
		spawn_block(&mut cmd, pos, display, mass);
	}
}
